@@ -28,10 +28,11 @@ def create_pipeline(torch_client):
 
 
 def create_datagen_job(pipeline):
+    pipeline_run = pipeline.get_latest_pipeline_run()
     job = CreateJob(
         uid='customer.order.datagen.job',
         name='customers and orders datagen',
-        pipelineSnapshot=pipeline.currentSnapshot,
+        version=pipeline_run.versionId,
         description='Generates Pseudo random data for Orders and Customers',
         inputs=[],
         outputs=[
@@ -48,10 +49,11 @@ def create_datagen_job(pipeline):
 
 
 def create_data_join_job(pipeline):
+    pipeline_run = pipeline.get_latest_pipeline_run()
     job = CreateJob(
         uid='customer.order.join.job',
         name='customers and orders joiner and Inserter',
-        pipelineSnapshot=pipeline.currentSnapshot,
+        version=pipeline_run.versionId,
         description='Joins the ',
         inputs=[Dataset('snowflake1647503515210',
                         'FINANCE.FINANCE.A1'), Dataset('snowflake1647503515210',
