@@ -1,4 +1,5 @@
-from enum import Enum
+from enum import Enum, IntEnum, auto
+
 
 class RuleExecutionStatus(Enum):
     STARTED = 1
@@ -9,5 +10,11 @@ class RuleExecutionStatus(Enum):
     ABORTED = 6
 
 
+class FailureStrategy(IntEnum):
+    DoNotFail = auto()
+    FailOnError = auto()
+    FailOnWarning = auto()
+
+
 DATA_QUALITY = 'DATA_QUALITY'
 RECONCILIATION = 'RECONCILIATION'
