@@ -17,23 +17,28 @@ class SpanContextStatus(Enum):
 
 class SpanContext:
 
-    def __init__(self, client=None, span: Span = None, parent=None, context_data= None):
+    def __init__(self, client=None, span: Span = None, parent=None, context_data= None, new_span: bool = None):
         """
         Description:
             SpanContext is used for instantiate event for given span.
         :param client: TorchClient class instance
         :param span: Span class instance
         :param parent: SpanContext class instance which parent of current span context
+        :param new_span: pass true value when its new span
         """
         self.client = client
         self.span = span
         self.parent = parent
         # self.status = SpanContextStatus.INITIALISED
         self.status = span.status
-        self.started = False
-        if self.status == SpanContextStatus.STARTED.name:
-            self.started = True
-        else:
+        self.started = True
+        if new_span is None:
+            new_span = False
+        self.new_span = new_span
+        # if self.status == SpanContextStatus.STARTED.name:
+        #     self.started = True
+        # else:
+        if new_span or self.status == SpanContextStatus.INITIALISED.name:
             self.start(context_data=context_data)
         self.children = []
 
@@ -41,7 +46,7 @@ class SpanContext:
         return f"SpanContext({self.__dict__})"
 
     def is_span_finished(self):
-        if self.span.status == SpanStatus.ABORTED.name or self.span.status == SpanStatus.FINISHED.name or self.span.status == SpanStatus.FAILED.name:
+        if self.span.status == SpanStatus.ABORTED.name or self.span.status == SpanStatus.FINISHED.name:
             logger.info('Can not send any event. Span is already completed')
             return True
         else:
@@ -131,10 +136,10 @@ class SpanContext:
         if self.is_span_finished():
             return
         if not self.started:
-            logger.info('Can not abort span that is not started yet')
+            logger.info('Can not fail span that is not started yet')
             return
         if self.status == SpanContextStatus.FAILED:
-            logger.info('Can not abort span that is aborted failed.')
+            logger.info('Can not fail span that is aborted failed.')
             return
         span_event = SpanEvent()
         event_res = span_event.convert(
@@ -185,9 +190,9 @@ class SpanContext:
         payload['status'] = create_child_span.status.name
         if associatedJobUids is None:
             associatedJobUids = []
-        child_span_payload = {'span': payload, 'associatedJobUids' : associatedJobUids}
+        child_span_payload = {'span': payload, 'associatedJobUids': associatedJobUids}
         res = self.client.create_span(self.span.pipelineRunId, child_span_payload)
-        span_context = SpanContext(client=self.client, span=res, parent=self, context_data= context_data)
+        span_context = SpanContext(client=self.client, span=res, parent=self, context_data= context_data, new_span= True)
         self.children.append(span_context)
         return span_context
 
@@ -197,7 +202,15 @@ class SpanContext:
             To check whether given spanContext has children or not.
         :return: True if it has children else False
         """
-        return len(self.children) > 0
+        return len(self.get_child_spans()) > 0
+
+    def get_child_spans(self):
+        """
+        Description:
+            To get child spans
+        :return: list of child spans
+        """
+        return self.client.get_child_spans(self.span.id)
 
     def is_root(self):
         """
