@@ -21,6 +21,9 @@ from torch_sdk.models.span import Span, SpanContextEvent, CreateSpanEvent
 from torch_sdk.client import TorchClientInterFace
 from torch_sdk.models.asset import Asset, AssetRelation, ChildAsset, Metadata
 from requests_toolbelt import MultipartEncoder
+from torch_sdk.models.dqrule import DataQualityExecutionResult
+from torch_sdk.models.reconcillationrule import ReconcillationExecutionResult
+from torch_sdk.models.ruleExecutionResult import RuleResult
 
 from dataclasses import asdict
 import json
@@ -79,7 +82,7 @@ class TorchHttpClient(TorchClientInterFace, ABC):
         return pipeline_dict
 
     # function to create pipeline
-    def create_pipeline(self, pipeline: CreatePipeline):
+    def create_pipeline(self, pipeline: CreatePipeline) -> Pipeline:
         """
         Description:
             Used to create pipeline
@@ -97,7 +100,7 @@ class TorchHttpClient(TorchClientInterFace, ABC):
         return Pipeline(**response['pipeline'])
 
     # get pipeline by uid
-    def get_pipeline(self, uid: str):
+    def get_pipeline(self, uid: str) -> Pipeline:
         """
             Description:
                 To get an existing pipeline from torch catalog
@@ -110,7 +113,7 @@ class TorchHttpClient(TorchClientInterFace, ABC):
         return Pipeline(**response['pipeline'])
 
     # to create job for any given pipeline
-    def create_job(self, job: {}, pipelineId: int):
+    def create_job(self, job: {}, pipelineId: int) -> Job:
         """
         Description:
             Used to create job in a pipeline
@@ -148,7 +151,7 @@ class TorchHttpClient(TorchClientInterFace, ABC):
         return datasource_payload
 
     # create datasource in torch
-    def create_datasource(self, datasource: CreateDataSource):
+    def create_datasource(self, datasource: CreateDataSource) -> DataSource:
         """
         Description:
             used to create datasource
@@ -163,7 +166,7 @@ class TorchHttpClient(TorchClientInterFace, ABC):
         response['data']['client'] = self
         return DataSource(**response['data'])
 
-    def update_datasource(self, datasource_id: int, datasource: CreateDataSource):
+    def update_datasource(self, datasource_id: int, datasource: CreateDataSource) -> DataSource:
         """
         Description:
             used to update datasource
@@ -202,7 +205,7 @@ class TorchHttpClient(TorchClientInterFace, ABC):
         return root_assets_list
 
     # to create asset for any given datasource
-    def create_asset(self, asset: {}):
+    def create_asset(self, asset: {}) -> Asset:
         """
         Description:
             Used to create asset in a datasource
@@ -216,34 +219,34 @@ class TorchHttpClient(TorchClientInterFace, ABC):
         response['data']['client'] = self
         return Asset(**response['data'])
 
-    def start_crawler(self, datasource_name):
+    def start_crawler(self, datasource_name) -> Crawler:
         response = self._post(
             f'{self._catalog_api_base}/crawler/{datasource_name}',
             payload={}
         )
         return Crawler(**response['data'])
 
-    def restart_crawler(self, datasource_name):
+    def restart_crawler(self, datasource_name) -> Crawler:
         response = self._post(
             f'{self._catalog_api_base}/crawler/{datasource_name}/restart',
             payload={}
         )
         return Crawler(**response['data'])
 
-    def remove_crawler(self, datasource_name):
+    def remove_crawler(self, datasource_name) -> Crawler:
         response = self._delete(
             f'{self._catalog_api_base}/crawler/{datasource_name}/restart'
         )
         return Crawler(**response['data'])
 
-    def get_crawler_status(self, datasource_name):
+    def get_crawler_status(self, datasource_name) -> Crawler:
         response = self._get(
             f'{self._catalog_api_base}/crawler/{datasource_name}',
         )
         return Crawler(**response['data'])
 
     # initialise new version of snapshot for a datasource
-    def initialise_snapshot(self, snapshot_data: {}):
+    def initialise_snapshot(self, snapshot_data: {}) -> SnapshotData:
         """
         Description:
             Used to initialise new version of snapshot for a datasource
@@ -257,7 +260,7 @@ class TorchHttpClient(TorchClientInterFace, ABC):
         return SnapshotData(**response['data'])
 
     # get current version of datasource
-    def get_current_snapshot(self, datasource_id: int):
+    def get_current_snapshot(self, datasource_id: int) -> SnapshotData:
         """
         Description:
             If you want to get current version of a datasource
@@ -290,7 +293,7 @@ class TorchHttpClient(TorchClientInterFace, ABC):
         return snapshots_list
 
     # create relation between asset
-    def create_asset_relation(self, asset_relation: {}):
+    def create_asset_relation(self, asset_relation: {}) -> AssetRelation:
         """
         Description:
             used to create relation between any 2 existing assets
@@ -304,7 +307,7 @@ class TorchHttpClient(TorchClientInterFace, ABC):
         return AssetRelation(**response['data'])
 
     # create run for a pipeline
-    def create_pipeline_run(self, pipeline_run: {}):
+    def create_pipeline_run(self, pipeline_run: {}) -> PipelineRun:
         """
         Description:
             used to create a pipeline run
@@ -321,7 +324,7 @@ class TorchHttpClient(TorchClientInterFace, ABC):
         return PipelineRun(**response['run'])
 
     # update run for a pipeline
-    def update_pipeline_run(self, pipeline_run_id: int, pipeline_run: {}):
+    def update_pipeline_run(self, pipeline_run_id: int, pipeline_run: {}) -> PipelineRun:
         """
         Description:
             used to update an existing pipeline run
@@ -338,7 +341,7 @@ class TorchHttpClient(TorchClientInterFace, ABC):
         return PipelineRun(**response['run'])
 
     # get latest pipeline run
-    def get_latest_pipeline_run(self, pipeline_id: int):
+    def get_latest_pipeline_run(self, pipeline_id: int) -> PipelineRun:
         """
             Description:
                 To get latest pipeline run instance of any pipeline
@@ -350,8 +353,21 @@ class TorchHttpClient(TorchClientInterFace, ABC):
         response['run']['client'] = self
         return PipelineRun(**response['run'])
 
+    # get pipeline run with particular id
+    def get_pipeline_run(self, pipeline_run_id: int) -> PipelineRun:
+        """
+            Description:
+                To get pipeline run with particular id of any pipeline
+        :param pipeline_run_id: id of the pipeline run
+        :return: PipelineRun instance
+        """
+        url = f'{self._pipeline_api_base}/pipelines/runs/{pipeline_run_id}'
+        response = self._get(url)
+        response['run']['client'] = self
+        return PipelineRun(**response['run'])
+
     # create span for pipeline run
-    def create_span(self, pipeline_run_id: int, span: dict):
+    def create_span(self, pipeline_run_id: int, span: dict) -> Span:
         """
         Description:
             used to create span for any pipeline run
@@ -377,7 +393,7 @@ class TorchHttpClient(TorchClientInterFace, ABC):
             child_spans_res.append(Span(**res))
         return child_spans_res
 
-    def get_span(self, pipeline_run_id: int, uid: str):
+    def get_span(self, pipeline_run_id: int, uid: str) -> Span:
         """
             Description:
                 Get span of the pipeline run by uid
@@ -390,7 +406,7 @@ class TorchHttpClient(TorchClientInterFace, ABC):
         response['span']['client'] = self
         return Span(**response['span'])
 
-    def get_root_span(self, pipeline_run_id: int):
+    def get_root_span(self, pipeline_run_id: int) -> Span:
         """
             Description:
                 Get root span of the pipeline run
@@ -420,7 +436,7 @@ class TorchHttpClient(TorchClientInterFace, ABC):
         return event_payload
 
     # create span event for any span
-    def create_span_event(self, span_event: CreateSpanEvent):
+    def create_span_event(self, span_event: CreateSpanEvent) -> SpanContextEvent:
         """
         Description:
             used to create span event
@@ -540,7 +556,7 @@ class TorchHttpClient(TorchClientInterFace, ABC):
         }
         return response
 
-    def get_datasource(self, name: str):
+    def get_datasource(self, name: str) -> DataSource:
         """
         Description:
             Find datasource by it's name in torch catalog
@@ -571,7 +587,7 @@ class TorchHttpClient(TorchClientInterFace, ABC):
             ds_list.append(DataSource(**res))
         return ds_list
 
-    def get_asset_by_id(self, id: int):
+    def get_asset_by_id(self, id: int) -> Asset:
         """
         Description:
             used to find an asset
@@ -936,7 +952,7 @@ class TorchHttpClient(TorchClientInterFace, ABC):
             comments_list.append(AssetComment(**res))
         return comments_list
 
-    def get_user_by_id(self, id: int):
+    def get_user_by_id(self, id: int) -> User:
         if id is None:
             raise TorchSdkException('User id is required')
         url = f'{self._catalog_api_base}/user/{id}'
@@ -945,7 +961,7 @@ class TorchHttpClient(TorchClientInterFace, ABC):
         )
         return User(**response)
 
-    def get_user_by_name(self, username: int):
+    def get_user_by_name(self, username: int) -> User:
         if id is None:
             raise TorchSdkException('Username is required')
         url = f'{self._catalog_api_base}/user/byname/{username}'
@@ -964,7 +980,7 @@ class TorchHttpClient(TorchClientInterFace, ABC):
             keys_list.append(AnalysisPipeline(**res))
         return keys_list
 
-    def get_analysis_pipeline(self, id: int):
+    def get_analysis_pipeline(self, id: int) -> AnalysisPipeline:
         response = self._get(
             url=f'{self._catalog_api_base}/analytics-pipelines/{id}'
         )
@@ -1008,7 +1024,7 @@ class TorchHttpClient(TorchClientInterFace, ABC):
                 return conn
         raise TorchSdkException(f'No connection found with name {name}')
 
-    def create_connection(self, create_connection):
+    def create_connection(self, create_connection) -> Connection:
         create_conn = create_connection.__dict__
         config_prop = []
         # for cp in create_connection.properties:
@@ -1024,7 +1040,7 @@ class TorchHttpClient(TorchClientInterFace, ABC):
         )
         return Connection(**response['connection'])
 
-    def update_connection(self, name, update_connection):
+    def update_connection(self, name, update_connection) -> Connection:
         connections = self.list_all_connections()
         conn_id = None
         for conn in connections:
@@ -1063,7 +1079,7 @@ class TorchHttpClient(TorchClientInterFace, ABC):
             auto_profile_list.append(AutoProfileConfiguration(**res))
         return auto_profile_list
 
-    def get_auto_profile_configuration(self, datasource_id: int):
+    def get_auto_profile_configuration(self, datasource_id: int) -> AutoProfileConfiguration:
         response = self._get(
             url=f'{self._catalog_api_base}/assemblies/{datasource_id}/auto-profile-configuration'
         )
@@ -1075,7 +1091,7 @@ class TorchHttpClient(TorchClientInterFace, ABC):
         )
         return response
 
-    def create_api_keys(self):
+    def create_api_keys(self) -> APIKeys:
         response = self._post(
             url=f'{self._catalog_api_base}/key'
         )
@@ -1124,7 +1140,7 @@ class TorchHttpClient(TorchClientInterFace, ABC):
             inc_list.append(Incident(**res))
         return inc_list
 
-    def enable_rule(self, rule_id):
+    def enable_rule(self, rule_id) -> Rule:
         payload = {
             "enable": True
         }
@@ -1134,7 +1150,7 @@ class TorchHttpClient(TorchClientInterFace, ABC):
         )
         return Rule(**response['rule'])
 
-    def disable_rule(self, rule_id):
+    def disable_rule(self, rule_id) -> Rule:
         payload = {
             "enable": False
         }
@@ -1144,14 +1160,14 @@ class TorchHttpClient(TorchClientInterFace, ABC):
         )
         return Rule(**response['rule'])
 
-    def cancel_rule_execution(self, execution_id):
+    def cancel_rule_execution(self, execution_id) -> RuleCancelResponse:
         response = self._put(
             url=f'{self._catalog_api_base}/rules/execution/{execution_id}/cancel',
             payload={}
         )
         return RuleCancelResponse(**response['result'])
 
-    def execute_dq_rule(self, rule_id, incremental=False):
+    def execute_dq_rule(self, rule_id, incremental=False) -> RuleExecution:
         payload_temp = {"executionType": "FULL"}
         if incremental:
             payload_temp = {"executionType": "INCREMENTAL"}
@@ -1161,7 +1177,7 @@ class TorchHttpClient(TorchClientInterFace, ABC):
             )
         return RuleExecution(**response['execution'])
 
-    def execute_reconciliation_rule(self, rule_id, incremental=False):
+    def execute_reconciliation_rule(self, rule_id, incremental=False) -> RuleExecution:
         payload_temp = {"executionType": "FULL"}
         if incremental:
             payload_temp = {"executionType": "INCREMENTAL"}
@@ -1171,214 +1187,144 @@ class TorchHttpClient(TorchClientInterFace, ABC):
         )
         return RuleExecution(**response['execution'])
 
-    def get_dq_rule_execution_details(self, execution_id):
+    def get_dq_rule_execution_details(self, execution_id) -> ExecutionResult:
         response = self._get(
             url=f'{self._catalog_api_base}/rules/data-quality/executions/{execution_id}'
         )
         return ExecutionResult(**response)
 
-    def get_reconciliation_rule_execution_details(self, execution_id):
+    def get_reconciliation_rule_execution_details(self, execution_id) -> ExecutionResult:
         response = self._get(
             url=f'{self._catalog_api_base}/rules/reconciliation/executions/{execution_id}'
         )
         return ExecutionResult(**response)
 
-    def get_reconciliation_rule_result(self, execution_id):
+    def get_reconciliation_rule_result(self, execution_id) -> RuleResult:
         response = self._get(
             url=f'{self._catalog_api_base}/rules/reconciliation/executions/{execution_id}/result'
         )
-        return ExecutionResult(**response)
+        return ReconcillationExecutionResult(**response)
 
-    def get_dq_rule_result(self, execution_id):
+    def get_dq_rule_result(self, execution_id) -> RuleResult:
         response = self._get(
             url=f'{self._catalog_api_base}/rules/data-quality/executions/{execution_id}/result'
         )
-        return ExecutionResult(**response)
+        return DataQualityExecutionResult(**response)
 
     @staticmethod
     def now_ms():
         return int(round(time.time() * 1000))
 
     def _post(self, url, payload=None, as_json=True):
-        retry = True
-        max_retry = 3
-        retry_count = 0
-        while retry:
-            now_ms = self.now_ms()
-            if _HEADERS['accessKey'] is None:
-                _HEADERS['accessKey'] = self.access_key
-            if _HEADERS['secretKey'] is None:
-                _HEADERS['secretKey'] = self.secret_key
-            response = requests.post(
-                url=url, headers=_HEADERS, json=payload, timeout=self._timeout, verify=False
-            )
-            self.logger.info(
-                f" POST {url} "
-                f"payload={json.dumps(payload)} "
-                f"duration_ms={self.now_ms() - now_ms}"
-            )
-            retry_count = retry_count + 1
-            if 500 <= response.status_code <= 511 and retry_count < max_retry:
-                retry = True
-                self.logger.info(
-                    f" Retrying POST call because of error code: {response.status_code}"
-                )
-            else:
-                retry = False
+        now_ms = self.now_ms()
+        if _HEADERS['accessKey'] is None:
+            _HEADERS['accessKey'] = self.access_key
+        if _HEADERS['secretKey'] is None:
+            _HEADERS['secretKey'] = self.secret_key
+        response = requests.post(
+            url=url, headers=_HEADERS, json=payload, timeout=self._timeout, verify=False
+        )
+        self.logger.info(
+            f" POST {url} "
+            f"payload={json.dumps(payload)} "
+            f"duration_ms={self.now_ms() - now_ms}"
+        )
+
         return self._response(response, as_json)
 
     def _put(self, url, payload=None, as_json=True):
-        retry = True
-        max_retry = 3
-        retry_count = 0
-        while retry:
-            now_ms = self.now_ms()
-            if _HEADERS['accessKey'] is None:
-                _HEADERS['accessKey'] = self.access_key
-            if _HEADERS['secretKey'] is None:
-                _HEADERS['secretKey'] = self.secret_key
-            response = requests.put(
-                url=url, headers=_HEADERS, json=payload, timeout=self._timeout, verify=False
-            )
-            self.logger.info(
-                f" PUT {url} "
-                f"payload={json.dumps(payload)} "
-                f"duration_ms={self.now_ms() - now_ms}"
-            )
-            retry_count = retry_count + 1
-            if 500 <= response.status_code <= 511 and retry_count < max_retry:
-                retry = True
-                self.logger.info(
-                    f" Retrying PUT call because of error code: {response.status_code}"
-                )
-            else:
-                retry = False
+        now_ms = self.now_ms()
+        if _HEADERS['accessKey'] is None:
+            _HEADERS['accessKey'] = self.access_key
+        if _HEADERS['secretKey'] is None:
+            _HEADERS['secretKey'] = self.secret_key
+        response = requests.put(
+            url=url, headers=_HEADERS, json=payload, timeout=self._timeout, verify=False
+        )
+        self.logger.info(
+            f" PUT {url} "
+            f"payload={json.dumps(payload)} "
+            f"duration_ms={self.now_ms() - now_ms}"
+        )
 
         return self._response(response, as_json)
 
     def _get(self, url, params=None, as_json=True):
-        retry = True
-        max_retry = 3
-        retry_count = 0
-        while retry:
-            now_ms = self.now_ms()
-            if _HEADERS['accessKey'] is None:
-                _HEADERS['accessKey'] = self.access_key
-            if _HEADERS['secretKey'] is None:
-                _HEADERS['secretKey'] = self.secret_key
-            response = requests.get(
-                url=url, params=params, headers=_HEADERS, timeout=self._timeout, verify=False
-            )
-            self.logger.info(
-                f" GET {url} "
-                f"duration_ms={self.now_ms() - now_ms}"
-            )
-            retry_count = retry_count + 1
-            if 500 <= response.status_code <= 511 and retry_count < max_retry:
-                retry = True
-                self.logger.info(
-                    f" Retrying GET call because of error code: {response.status_code}"
-                )
-            else:
-                retry = False
+        now_ms = self.now_ms()
+        if _HEADERS['accessKey'] is None:
+            _HEADERS['accessKey'] = self.access_key
+        if _HEADERS['secretKey'] is None:
+            _HEADERS['secretKey'] = self.secret_key
+        response = requests.get(
+            url=url, params=params, headers=_HEADERS, timeout=self._timeout, verify=False
+        )
+        self.logger.info(
+            f" GET {url} "
+            f"duration_ms={self.now_ms() - now_ms}"
+        )
 
         return self._response(response, as_json)
 
     def _post_form_data(self, url, payload=None, as_json=True):
-        retry = True
-        max_retry = 3
-        retry_count = 0
-        while retry:
-            now_ms = self.now_ms()
-            if _HEADERS['accessKey'] is None:
-                _HEADERS['accessKey'] = self.access_key
-            if _HEADERS['secretKey'] is None:
-                _HEADERS['secretKey'] = self.secret_key
+        now_ms = self.now_ms()
+        if _HEADERS['accessKey'] is None:
+            _HEADERS['accessKey'] = self.access_key
+        if _HEADERS['secretKey'] is None:
+            _HEADERS['secretKey'] = self.secret_key
 
-            params = {
-                'request': json.dumps(payload)
-            }
-            data = MultipartEncoder(fields=params)
-            _HEADERS['Content-Type'] = data.content_type
-            response = requests.post(
-                url=url, headers=_HEADERS, data=data, timeout=self._timeout, verify=False
-            )
-            self.logger.info(
-                f" POST {url} "
-                f"payload={json.dumps(payload)} "
-                f"duration_ms={self.now_ms() - now_ms}"
-            )
-            retry_count = retry_count + 1
-            if 500 <= response.status_code <= 511 and retry_count < max_retry:
-                retry = True
-                self.logger.info(
-                    f" Retrying POST call because of error code: {response.status_code}"
-                )
-            else:
-                retry = False
+        params = {
+            'request': json.dumps(payload)
+        }
+        data = MultipartEncoder(fields=params)
+        _HEADERS['Content-Type'] = data.content_type
+        response = requests.post(
+            url=url, headers=_HEADERS, data=data, timeout=self._timeout, verify=False
+        )
+        self.logger.info(
+            f" POST {url} "
+            f"payload={json.dumps(payload)} "
+            f"duration_ms={self.now_ms() - now_ms}"
+        )
 
         return self._response(response, as_json)
 
     def _put_form_data(self, url, payload=None, as_json=True):
-        retry = True
-        max_retry = 3
-        retry_count = 0
-        while retry:
-            now_ms = self.now_ms()
-            if _HEADERS['accessKey'] is None:
-                _HEADERS['accessKey'] = self.access_key
-            if _HEADERS['secretKey'] is None:
-                _HEADERS['secretKey'] = self.secret_key
+        now_ms = self.now_ms()
+        if _HEADERS['accessKey'] is None:
+            _HEADERS['accessKey'] = self.access_key
+        if _HEADERS['secretKey'] is None:
+            _HEADERS['secretKey'] = self.secret_key
 
-            params = {
-                'request': json.dumps(payload)
-            }
-            data = MultipartEncoder(fields=params)
-            _HEADERS['Content-Type'] = data.content_type
-            response = requests.put(
-                url=url, headers=_HEADERS, data=data, timeout=self._timeout, verify=False
-            )
-            self.logger.info(
-                f" POSPUTT {url} "
-                f"payload={json.dumps(payload)} "
-                f"duration_ms={self.now_ms() - now_ms}"
-            )
-            retry_count = retry_count + 1
-            if 500 <= response.status_code <= 511 and retry_count < max_retry:
-                retry = True
-                self.logger.info(
-                    f" Retrying POSPUTT call because of error code: {response.status_code}"
-                )
-            else:
-                retry = False
+        params = {
+            'request': json.dumps(payload)
+        }
+        data = MultipartEncoder(fields=params)
+        _HEADERS['Content-Type'] = data.content_type
+        response = requests.put(
+            url=url, headers=_HEADERS, data=data, timeout=self._timeout, verify=False
+        )
+        self.logger.info(
+            f" POSPUTT {url} "
+            f"payload={json.dumps(payload)} "
+            f"duration_ms={self.now_ms() - now_ms}"
+        )
 
         return self._response(response, as_json)
 
     def _delete(self, url, params=None, as_json=True):
-        retry = True
-        max_retry = 3
-        retry_count = 0
-        while retry:
-            now_ms = self.now_ms()
-            if _HEADERS['accessKey'] is None:
-                _HEADERS['accessKey'] = self.access_key
-            if _HEADERS['secretKey'] is None:
-                _HEADERS['secretKey'] = self.secret_key
-            response = requests.delete(
-                url=url, params=params, headers=_HEADERS, timeout=self._timeout, verify=False
-            )
-            self.logger.info(
-                f" DELETE {url} "
-                f"duration_ms={self.now_ms() - now_ms}"
-            )
-            retry_count = retry_count + 1
-            if 500 <= response.status_code <= 511 and retry_count < max_retry:
-                retry = True
-                self.logger.info(
-                    f" Retrying DELETE call because of error code: {response.status_code}"
-                )
-            else:
-                retry = False
+        now_ms = self.now_ms()
+        if _HEADERS['accessKey'] is None:
+            _HEADERS['accessKey'] = self.access_key
+        if _HEADERS['secretKey'] is None:
+            _HEADERS['secretKey'] = self.secret_key
+        response = requests.delete(
+            url=url, params=params, headers=_HEADERS, timeout=self._timeout, verify=False
+        )
+        self.logger.info(
+            f" DELETE {url} "
+            f"duration_ms={self.now_ms() - now_ms}"
+        )
+
         return response
 
     def _response(self, response, as_json):
@@ -1386,9 +1332,9 @@ class TorchHttpClient(TorchClientInterFace, ABC):
             response.raise_for_status()
         except requests.exceptions.HTTPError as e:
             self.logger.error(f' {response.text}')
-            self._raise_api_error(e)
+            self._raise_api_error(e, response.text)
 
         return response.json() if as_json else response.text
 
-    def _raise_api_error(self, e):
-        raise APIError() from e
+    def _raise_api_error(self, e, text):
+        raise APIError(text) from e
